/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEIB
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2017 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file ipa_point_to_analysis.hpp
 * @brief Perform an inter-procedural flow sensitive point-to analysis.
 *
 * @author Fabrizio Ferrandi <fabrizio.ferrandi@polimi.it>
 * $Revision$
 * $Date$ $
 * Last modified by $Author$
 *
*/

#ifndef IPA_POINT_TO_ANALYSIS_HPP
#define IPA_POINT_TO_ANALYSIS_HPP

///Superclass include
#include "application_frontend_flow_step.hpp"

///Utility include
#include "refcount.hpp"
#include <list>

CONSTREF_FORWARD_DECL(tree_manager);

class ipa_point_to_analysis : public ApplicationFrontendFlowStep
{
   private:

      ///The tree manager
      const tree_managerConstRef TM;

      /// compute the topological order of the functions called by the top function
      void compute_function_topological_order(std::list<unsigned int> &sort_list);


      /**
       * Return the set of analyses in relationship with this design step
       * @param relationship_type is the type of relationship to be considered
       */
      const std::unordered_set<std::pair<FrontendFlowStepType, FunctionRelationship> > ComputeFrontendRelationships(const DesignFlowStep::RelationshipType relationship_type) const;

   public:
      /**
       * Constructor.
       * @param AppM is the application manager
       * @param design_flow_manager is the design flow manager
       * @param parameters is the set of input parameters
       */
      ipa_point_to_analysis(const application_managerRef AppM, const DesignFlowManagerConstRef design_flow_manager, const ParameterConstRef parameters);

      /**
       *  Destructor
       */
      ~ipa_point_to_analysis();

      /**
       * Determines the variables that require a memory access
       */
      DesignFlowStep_Status Exec();

};
#endif
